@@ -1,6 +1,5 @@
 /* md.c  -  message digest dispatcher
- * Copyright (C) 1998, 1999, 2002, 2003, 2006,
- *               2008 Free Software Foundation, Inc.
+ * Copyright (C) 1998, 1999, 2002, 2003 Free Software Foundation, Inc.
  *
  * This file is part of Libgcrypt.
  *
@@ -15,7 +14,8 @@
  * GNU Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this program; if not, see <http://www.gnu.org/licenses/>.
+ * License along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 
 #include <config.h>
@@ -23,6 +23,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <errno.h>
+#include <assert.h>
 
 #include "g10lib.h"
 #include "cipher.h"
@@ -30,67 +31,38 @@
 
 #include "rmd.h"
 
-/* A dummy extraspec so that we do not need to tests the extraspec
-   field from the module specification against NULL and instead
-   directly test the respective fields of extraspecs.  */
-static md_extra_spec_t dummy_extra_spec;
-
-
-/* This is the list of the digest implementations included in
-   libgcrypt.  */
 static struct digest_table_entry
 {
   gcry_md_spec_t *digest;
-  md_extra_spec_t *extraspec;
   unsigned int algorithm;
-  int fips_allowed;
 } digest_table[] =
   {
 #if USE_CRC    
-    /* We allow the CRC algorithms even in FIPS mode because they are
-       actually no cryptographic primitives.  */
-    { &_gcry_digest_spec_crc32,   
-      &dummy_extra_spec,                 GCRY_MD_CRC32, 1 },
-    { &_gcry_digest_spec_crc32_rfc1510,  
-      &dummy_extra_spec,                 GCRY_MD_CRC32_RFC1510, 1 },
-    { &_gcry_digest_spec_crc24_rfc2440,
-      &dummy_extra_spec,                 GCRY_MD_CRC24_RFC2440, 1 },
+    { &_gcry_digest_spec_crc32, GCRY_MD_CRC32 },
+    { &_gcry_digest_spec_crc32_rfc1510, GCRY_MD_CRC32_RFC1510 },
+    { &_gcry_digest_spec_crc24_rfc2440, GCRY_MD_CRC24_RFC2440 },
 #endif
 #if USE_MD4
-    { &_gcry_digest_spec_md4,
-      &dummy_extra_spec,                 GCRY_MD_MD4 },
+    { &_gcry_digest_spec_md4, GCRY_MD_MD4 },
 #endif
 #if USE_MD5
-    { &_gcry_digest_spec_md5,
-      &dummy_extra_spec,                 GCRY_MD_MD5 },
+    { &_gcry_digest_spec_md5, GCRY_MD_MD5 },
 #endif
 #if USE_RMD160
-    { &_gcry_digest_spec_rmd160,
-      &dummy_extra_spec,                 GCRY_MD_RMD160 },
+    { &_gcry_digest_spec_rmd160, GCRY_MD_RMD160 },
 #endif
 #if USE_SHA1
-    { &_gcry_digest_spec_sha1, 
-      &_gcry_digest_extraspec_sha1,      GCRY_MD_SHA1, 1 },
+    { &_gcry_digest_spec_sha1, GCRY_MD_SHA1 },
 #endif
 #if USE_SHA256
-    { &_gcry_digest_spec_sha256,
-      &_gcry_digest_extraspec_sha256,    GCRY_MD_SHA256, 1 },
-    { &_gcry_digest_spec_sha224,
-      &_gcry_digest_extraspec_sha224,    GCRY_MD_SHA224, 1 },
+    { &_gcry_digest_spec_sha256, GCRY_MD_SHA256 },
 #endif
 #if USE_SHA512
-    { &_gcry_digest_spec_sha512,
-      &_gcry_digest_extraspec_sha512,    GCRY_MD_SHA512, 1 },
-    { &_gcry_digest_spec_sha384,
-      &_gcry_digest_extraspec_sha384,    GCRY_MD_SHA384, 1 },
+    { &_gcry_digest_spec_sha512, GCRY_MD_SHA512 },
+    { &_gcry_digest_spec_sha384, GCRY_MD_SHA384 },
 #endif
 #if USE_TIGER
-    { &_gcry_digest_spec_tiger,
-      &dummy_extra_spec,                 GCRY_MD_TIGER },
-#endif
-#if USE_WHIRLPOOL
-    { &_gcry_digest_spec_whirlpool,
-      &dummy_extra_spec,                 GCRY_MD_WHIRLPOOL },
+    { &_gcry_digest_spec_tiger, GCRY_MD_TIGER },
 #endif
     { NULL },
   };
@@ -125,7 +97,6 @@ struct gcry_md_context
   int finalized;
   GcryDigestEntry *list;
   byte *macpads;
-  int macpads_Bsize;             /* Blocksize as used for the HMAC pads. */
 };
 
 
@@ -139,7 +110,7 @@ struct gcry_md_context
       ath_mutex_lock (&digests_registered_lock);   \
       if (! default_digests_registered)            \
         {                                          \
-          md_register_default ();                  \
+          gcry_md_register_default ();             \
           default_digests_registered = 1;          \
         }                                          \
       ath_mutex_unlock (&digests_registered_lock); \
@@ -154,14 +125,14 @@ static gcry_err_code_t md_open (gcry_md_hd_t *h, int algo,
 static gcry_err_code_t md_enable (gcry_md_hd_t hd, int algo);
 static gcry_err_code_t md_copy (gcry_md_hd_t a, gcry_md_hd_t *b);
 static void md_close (gcry_md_hd_t a);
-static void md_write (gcry_md_hd_t a, const void *inbuf, size_t inlen);
+static void md_write (gcry_md_hd_t a, byte *inbuf, size_t inlen);
 static void md_final(gcry_md_hd_t a);
 static byte *md_read( gcry_md_hd_t a, int algo );
 static int md_get_algo( gcry_md_hd_t a );
 static int md_digest_length( int algo );
 static const byte *md_asn_oid( int algo, size_t *asnlen, size_t *mdlen );
-static void md_start_debug ( gcry_md_hd_t a, const char *suffix );
-static void md_stop_debug ( gcry_md_hd_t a );
+static void md_start_debug( gcry_md_hd_t a, char *suffix );
+static void md_stop_debug( gcry_md_hd_t a );
 
 
 
@@ -169,22 +140,16 @@ static void md_stop_debug ( gcry_md_hd_t a );
 /* Internal function.  Register all the ciphers included in
    CIPHER_TABLE.  Returns zero on success or an error code.  */
 static void
-md_register_default (void)
+gcry_md_register_default (void)
 {
   gcry_err_code_t err = 0;
   int i;
   
-  for (i = 0; !err && digest_table[i].digest; i++)
-    {
-      if ( fips_mode () && !digest_table[i].fips_allowed )
-        continue;
-
-      err = _gcry_module_add (&digests_registered,
-                              digest_table[i].algorithm,
-                              (void *) digest_table[i].digest,
-                              (void *) digest_table[i].extraspec,
-                              NULL);
-    }
+  for (i = 0; (! err) && digest_table[i].digest; i++)
+    err = _gcry_module_add (&digests_registered,
+			    digest_table[i].algorithm,
+			    (void *) digest_table[i].digest,
+			    NULL);
 
   if (err)
     BUG ();
@@ -247,23 +212,16 @@ gcry_md_lookup_oid (const char *oid)
    DIGEST.  On success, a new algorithm ID is stored in ALGORITHM_ID
    and a pointer representhing this module is stored in MODULE.  */
 gcry_error_t
-_gcry_md_register (gcry_md_spec_t *digest,
-                   md_extra_spec_t *extraspec,
-                   unsigned int *algorithm_id,
-                   gcry_module_t *module)
+gcry_md_register (gcry_md_spec_t *digest,
+		  unsigned int *algorithm_id,
+		  gcry_module_t *module)
 {
   gcry_err_code_t err = 0;
   gcry_module_t mod;
 
-  /* We do not support module loading in fips mode.  */
-  if (fips_mode ())
-    return gpg_error (GPG_ERR_NOT_SUPPORTED);
-
   ath_mutex_lock (&digests_registered_lock);
   err = _gcry_module_add (&digests_registered, 0,
-			  (void *) digest, 
-			  (void *)(extraspec? extraspec : &dummy_extra_spec), 
-                          &mod);
+			  (void *) digest, &mod);
   ath_mutex_unlock (&digests_registered_lock);
   
   if (! err)
@@ -340,7 +298,7 @@ gcry_md_map_name (const char *string)
   ret = search_oid (string, &algorithm, NULL);
   if (! ret)
     {
-      /* Not found, search a matching digest name.  */
+      /* Not found, search for an acording diget name.  */
       digest = gcry_md_lookup_name (string);
       if (digest)
 	{
@@ -468,24 +426,18 @@ md_open (gcry_md_hd_t *h, int algo, int secure, int hmac)
 
       if (hmac)
 	{
-	  if ( (GCRY_MD_SHA384 == algo) || (GCRY_MD_SHA512 == algo) ) {
-	    ctx->macpads_Bsize = 128;
-	    ctx->macpads = gcry_malloc_secure (2*(ctx->macpads_Bsize));
-	  } else {
-	    ctx->macpads_Bsize = 64;
-	    ctx->macpads = gcry_malloc_secure (2*(ctx->macpads_Bsize));
-	  }
+	  ctx->macpads = gcry_malloc_secure (128);
 	  if (! ctx->macpads)
 	    {
-	      err = gpg_err_code_from_errno (errno);
 	      md_close (hd);
+	      err = gpg_err_code_from_errno (errno);
 	    }
 	}
     }
 
   if (! err)
     {
-      /* Hmmm, should we really do that? - yes [-wk] */
+      /* FIXME: should we really do that? - yes [-wk] */
       _gcry_fast_random_poll ();
 
       if (algo)
@@ -597,7 +549,8 @@ md_enable (gcry_md_hd_t hd, int algorithm)
 gcry_error_t
 gcry_md_enable (gcry_md_hd_t hd, int algorithm)
 {
-  return gcry_error (md_enable (hd, algorithm));
+  gcry_err_code_t err = md_enable (hd, algorithm);
+  return gcry_error (err);
 }
 
 static gcry_err_code_t
@@ -626,63 +579,54 @@ md_copy (gcry_md_hd_t ahd, gcry_md_hd_t *b_hd)
     {
       bhd->ctx = b = (struct gcry_md_context *) ((char *) bhd + n);
       /* No need to copy the buffer due to the write above. */
-      gcry_assert (ahd->bufsize == (n - sizeof (struct gcry_md_handle) + 1));
+      assert (ahd->bufsize == (n - sizeof (struct gcry_md_handle) + 1));
       bhd->bufsize = ahd->bufsize;
       bhd->bufpos = 0;
-      gcry_assert (! ahd->bufpos);
+      assert (! ahd->bufpos);
       memcpy (b, a, sizeof *a);
       b->list = NULL;
       b->debug = NULL;
       if (a->macpads)
 	{
-	  b->macpads = gcry_malloc_secure (2*(a->macpads_Bsize));
+	  b->macpads = gcry_malloc_secure (128);
 	  if (! b->macpads)
 	    {
-	      err = gpg_err_code_from_errno (errno);
 	      md_close (bhd);
+	      err = gpg_err_code_from_errno (errno);
 	    }
 	  else
-	    memcpy (b->macpads, a->macpads, (2*(a->macpads_Bsize)));
+	    memcpy (b->macpads, a->macpads, 128);
 	}
     }
 
   /* Copy the complete list of algorithms.  The copied list is
      reversed, but that doesn't matter. */
-  if (!err)
-    {
-      for (ar = a->list; ar; ar = ar->next)
-        {
-          if (a->secure)
-            br = gcry_malloc_secure (sizeof *br
-                                     + ar->digest->contextsize
-                                     - sizeof(ar->context));
-          else
-            br = gcry_malloc (sizeof *br
-                              + ar->digest->contextsize
-                              - sizeof (ar->context));
-          if (!br)
-            {
-	      err = gpg_err_code_from_errno (errno);
-              md_close (bhd);
-              break;
-            }
+  if (! err)
+    for (ar = a->list; ar; ar = ar->next)
+      {
+	if (a->secure)
+	  br = gcry_xmalloc_secure (sizeof *br
+				    + ar->digest->contextsize
+				    - sizeof(ar->context));
+	else
+	  br = gcry_xmalloc (sizeof *br
+			     + ar->digest->contextsize
+			     - sizeof (ar->context));
+	memcpy (br, ar,
+		sizeof (*br) + ar->digest->contextsize - sizeof (ar->context));
+	br->next = b->list;
+	b->list = br;
 
-          memcpy (br, ar, (sizeof (*br) + ar->digest->contextsize
-                           - sizeof (ar->context)));
-          br->next = b->list;
-          b->list = br;
-          
-          /* Add a reference to the module.  */
-          ath_mutex_lock (&digests_registered_lock);
-          _gcry_module_use (br->module);
-          ath_mutex_unlock (&digests_registered_lock);
-        }
-    }
+	/* Add a reference to the module.  */
+	ath_mutex_lock (&digests_registered_lock);
+	_gcry_module_use (br->module);
+	ath_mutex_unlock (&digests_registered_lock);
+       }
 
-  if (a->debug && !err)
+  if (a->debug)
     md_start_debug (bhd, "unknown");
 
-  if (!err)
+  if (! err)
     *b_hd = bhd;
 
   return err;
@@ -691,9 +635,7 @@ md_copy (gcry_md_hd_t ahd, gcry_md_hd_t *b_hd)
 gcry_error_t
 gcry_md_copy (gcry_md_hd_t *handle, gcry_md_hd_t hd)
 {
-  gcry_err_code_t err;
-
-  err = md_copy (hd, handle);
+  gcry_err_code_t err = md_copy (hd, handle);
   if (err)
     *handle = NULL;
   return gcry_error (err);
@@ -707,9 +649,7 @@ void
 gcry_md_reset (gcry_md_hd_t a)
 {
   GcryDigestEntry *r;
-
-  /* Note: We allow this even in fips non operational mode.  */
-
+  
   a->bufpos = a->ctx->finalized = 0;
 
   for (r = a->ctx->list; r; r = r->next)
@@ -718,7 +658,7 @@ gcry_md_reset (gcry_md_hd_t a)
       (*r->digest->init) (&r->context.c);
     }
   if (a->ctx->macpads)
-    md_write (a, a->ctx->macpads, a->ctx->macpads_Bsize); /* inner pad */
+    md_write (a, a->ctx->macpads, 64); /* inner pad */
 }
 
 static void
@@ -742,7 +682,7 @@ md_close (gcry_md_hd_t a)
 
   if (a->ctx->macpads)
     {
-      wipememory (a->ctx->macpads, 2*(a->ctx->macpads_Bsize));
+      wipememory (a->ctx->macpads, 128);
       gcry_free(a->ctx->macpads);
     }
 
@@ -753,12 +693,11 @@ md_close (gcry_md_hd_t a)
 void
 gcry_md_close (gcry_md_hd_t hd)
 {
-  /* Note: We allow this even in fips non operational mode.  */
   md_close (hd);
 }
 
 static void
-md_write (gcry_md_hd_t a, const void *inbuf, size_t inlen)
+md_write (gcry_md_hd_t a, byte *inbuf, size_t inlen)
 {
   GcryDigestEntry *r;
   
@@ -782,7 +721,7 @@ md_write (gcry_md_hd_t a, const void *inbuf, size_t inlen)
 void
 gcry_md_write (gcry_md_hd_t hd, const void *inbuf, size_t inlen)
 {
-  md_write (hd, inbuf, inlen);
+  md_write (hd, (unsigned char *) inbuf, inlen);
 }
 
 static void
@@ -812,9 +751,7 @@ md_final (gcry_md_hd_t a)
 
       if (err)
 	_gcry_fatal_error (err, NULL);
-      md_write (om, 
-                (a->ctx->macpads)+(a->ctx->macpads_Bsize), 
-                a->ctx->macpads_Bsize);
+      md_write (om, a->ctx->macpads+64, 64);
       md_write (om, p, dlen);
       md_final (om);
       /* Replace our digest with the mac (they have the same size). */
@@ -842,15 +779,15 @@ prepare_macpads( gcry_md_hd_t hd, const byte *key, size_t keylen)
       gcry_md_hash_buffer ( algo, helpkey, key, keylen );
       key = helpkey;
       keylen = md_digest_length( algo );
-      gcry_assert ( keylen <= 64 );
+      assert ( keylen <= 64 );
     }
 
-  memset ( hd->ctx->macpads, 0, 2*(hd->ctx->macpads_Bsize) );
+  memset ( hd->ctx->macpads, 0, 128 );
   ipad = hd->ctx->macpads;
-  opad = (hd->ctx->macpads)+(hd->ctx->macpads_Bsize);
+  opad = hd->ctx->macpads+64;
   memcpy ( ipad, key, keylen );
   memcpy ( opad, key, keylen );
-  for (i=0; i < (hd->ctx->macpads_Bsize); i++ ) 
+  for (i=0; i < 64; i++ ) 
     {
       ipad[i] ^= 0x36;
       opad[i] ^= 0x5c;
@@ -861,7 +798,7 @@ prepare_macpads( gcry_md_hd_t hd, const byte *key, size_t keylen)
 }
 
 gcry_error_t
-gcry_md_ctl (gcry_md_hd_t hd, int cmd, void *buffer, size_t buflen)
+gcry_md_ctl (gcry_md_hd_t hd, int cmd, byte *buffer, size_t buflen)
 {
   gcry_err_code_t rc = 0;
   
@@ -877,7 +814,7 @@ gcry_md_ctl (gcry_md_hd_t hd, int cmd, void *buffer, size_t buflen)
       md_start_debug (hd, buffer);
       break;
     case GCRYCTL_STOP_DUMP:
-      md_stop_debug ( hd );
+      md_stop_debug( hd );
       break;
     default:
       rc = GPG_ERR_INV_OP;
@@ -890,7 +827,7 @@ gcry_md_setkey (gcry_md_hd_t hd, const void *key, size_t keylen)
 {
   gcry_err_code_t rc = GPG_ERR_NO_ERROR;
 
-  if (!hd->ctx->macpads)
+  if (! hd->ctx->macpads)
     rc = GPG_ERR_CONFLICT;
   else
     {
@@ -902,19 +839,6 @@ gcry_md_setkey (gcry_md_hd_t hd, const void *key, size_t keylen)
   return gcry_error (rc);
 }
 
-/* The new debug interface.  If SUFFIX is a string it creates an debug
-   file for the context HD.  IF suffix is NULL, the file is closed and
-   debugging is stopped.  */
-void
-gcry_md_debug (gcry_md_hd_t hd, const char *suffix)
-{
-  if (suffix)
-    md_start_debug (hd, suffix);
-  else
-    md_stop_debug (hd);
-}
-
-
 
 /****************
  * if ALGO is null get the digest for the used algo (which should be only one)
@@ -928,7 +852,7 @@ md_read( gcry_md_hd_t a, int algo )
     {
       /* return the first algorithm */
       if (r && r->next)
-	log_debug ("more than one algorithm in md_read(0)\n");
+	log_debug("more than algorithm in md_read(0)\n");
       return r->digest->read( &r->context.c );
     }
   else
@@ -948,27 +872,69 @@ md_read( gcry_md_hd_t a, int algo )
 byte *
 gcry_md_read (gcry_md_hd_t hd, int algo)
 {
-  /* This function is expected to always return a digest, thus we
-     can't return an error which we actually should do in
-     non-operational state.  */
   gcry_md_ctl (hd, GCRYCTL_FINALIZE, NULL, 0);
   return md_read (hd, algo);
 }
 
+/****************
+ * This function combines md_final and md_read but keeps the context
+ * intact.  This function can be used to calculate intermediate
+ * digests.  The digest is copied into buffer and the digestlength is
+ * returned.  If buffer is NULL only the needed size for buffer is returned.
+ * buflen gives the max size of buffer. If the buffer is too shourt to
+ * hold the complete digest, the buffer is filled with as many bytes are
+ * possible and this value is returned.
+ */
+#if 0
+static int
+md_digest( gcry_md_hd_t a, int algo, byte *buffer, int buflen )
+{
+  struct md_digest_list_s *r = NULL;
+  char *context;
+  char *digest;
+
+  if( a->bufpos )
+    md_write( a, NULL, 0 );
+
+  if( !algo ) {  /* return digest for the first algorithm */
+    if( (r=a->ctx->list) && r->next )
+      log_debug("more than algorithm in md_digest(0)\n");
+  }
+  else {
+    for(r=a->ctx->list; r; r = r->next )
+      if( r->algo == algo )
+	break;
+  }
+  if( !r )
+    BUG();
+
+  if( !buffer )
+    return r->mdlen;
+
+  /* I don't want to change the interface, so I simply work on a copy
+   * of the context (extra overhead - should be fixed)*/
+  context = a->ctx->secure ? gcry_xmalloc_secure( r->contextsize )
+    : gcry_xmalloc( r->contextsize );
+  memcpy( context, r->context.c, r->contextsize );
+  (*r->digest->final)( context );
+  digest = (*r->digest->read)( context );
+
+  if( buflen > r->mdlen )
+    buflen = r->mdlen;
+  memcpy( buffer, digest, buflen );
+
+  gcry_free(context);
+  return buflen;
+}
+#endif
 
 /*
- * Read out an intermediate digest.  Not yet functional.
+ * Read out an intermediate digest.  Not yet fucntional.
  */
 gcry_err_code_t
 gcry_md_get (gcry_md_hd_t hd, int algo, byte *buffer, int buflen)
 {
-  (void)hd;
-  (void)algo;
-  (void)buffer;
-  (void)buflen;
-
   /*md_digest ... */
-  fips_signal_error ("unimplemented function called");
   return GPG_ERR_INTERNAL;
 }
 
@@ -985,7 +951,7 @@ gcry_md_hash_buffer (int algo, void *digest,
 {
   if (algo == GCRY_MD_SHA1)
     _gcry_sha1_hash_buffer (digest, buffer, length);
-  else if (algo == GCRY_MD_RMD160 && !fips_mode () )
+  else if (algo == GCRY_MD_RMD160)
     _gcry_rmd160_hash_buffer (digest, buffer, length);
   else
     {
@@ -1009,10 +975,7 @@ md_get_algo (gcry_md_hd_t a)
   GcryDigestEntry *r = a->ctx->list;
 
   if (r && r->next)
-    {
-      fips_signal_error ("possible usage error");
-      log_error ("WARNING: more than one algorithm in md_get_algo()\n");
-    }
+    log_error("WARNING: more than algorithm in md_get_algo()\n");
   return r ? r->module->mod_id : 0;
 }
 
@@ -1118,31 +1081,27 @@ gcry_md_algo_info (int algo, int what, void *buffer, size_t *nbytes)
       break;
 
     case GCRYCTL_GET_ASNOID:
-      /* We need to check that the algo is available because
-         md_asn_oid would otherwise raise an assertion. */
-      err = check_digest_algo (algo);
-      if (!err)
-        {
-          const char unsigned *asn;
-          size_t asnlen;
-          
-          asn = md_asn_oid (algo, &asnlen, NULL);
-          if (buffer && (*nbytes >= asnlen))
+      {
+	const char unsigned *asn;
+	size_t asnlen;
+
+	asn = md_asn_oid (algo, &asnlen, NULL);
+	if (buffer && (*nbytes >= asnlen))
 	  {
 	    memcpy (buffer, asn, asnlen);
 	    *nbytes = asnlen;
 	  }
-          else if (!buffer && nbytes)
-            *nbytes = asnlen;
-          else
-            {
-              if (buffer)
-                err = GPG_ERR_TOO_SHORT;
-              else
-                err = GPG_ERR_INV_ARG;
-            }
-        }
-      break;
+	else if ((! buffer) && nbytes)
+	  *nbytes = asnlen;
+	else
+	  {
+	    if (buffer)
+	      err = GPG_ERR_TOO_SHORT;
+	    else
+	      err = GPG_ERR_INV_ARG;
+	  }
+	break;
+      }
 
   default:
     err = GPG_ERR_INV_OP;
@@ -1153,37 +1112,31 @@ gcry_md_algo_info (int algo, int what, void *buffer, size_t *nbytes)
 
 
 static void
-md_start_debug ( gcry_md_hd_t md, const char *suffix )
+md_start_debug( gcry_md_hd_t md, char *suffix )
 {
   static int idx=0;
   char buf[50];
 
-  if (fips_mode ())
+  if( md->ctx->debug ) {
+    log_debug("Oops: md debug already started\n");
     return;
-  
-  if ( md->ctx->debug )
-    {
-      log_debug("Oops: md debug already started\n");
-      return;
-    }
+  }
   idx++;
-  snprintf (buf, DIM(buf)-1, "dbgmd-%05d.%.10s", idx, suffix );
+  sprintf(buf, "dbgmd-%05d.%.10s", idx, suffix );
   md->ctx->debug = fopen(buf, "w");
-  if ( !md->ctx->debug )
+  if( !md->ctx->debug )
     log_debug("md debug: can't open %s\n", buf );
 }
 
 static void
 md_stop_debug( gcry_md_hd_t md )
 {
-  if ( md->ctx->debug )
-    {
-      if ( md->bufpos )
-        md_write ( md, NULL, 0 );
-      fclose (md->ctx->debug);
-      md->ctx->debug = NULL;
-    }
-
+  if( md->ctx->debug ) {
+    if( md->bufpos )
+      md_write( md, NULL, 0 );
+    fclose(md->ctx->debug);
+    md->ctx->debug = NULL;
+  }
 #ifdef HAVE_U64_TYPEDEF
   {  /* a kludge to pull in the __muldi3 for Solaris */
     volatile u32 a = (u32)(ulong)md;
@@ -1202,7 +1155,7 @@ md_stop_debug( gcry_md_hd_t md )
  *	Returns 1 when the handle works on secured memory
  *	otherwise 0 is returned.  There is no error return.
  *  GCRYCTL_IS_ALGO_ENABLED:
- *     Returns 1 if the algo is enabled for that handle.
+ *     Returns 1 if the algo is enanled for that handle.
  *     The algo must be passed as the address of an int.
  */
 gcry_error_t
@@ -1246,8 +1199,6 @@ gcry_md_info (gcry_md_hd_t h, int cmd, void *buffer, size_t *nbytes)
   return gcry_error (err);
 }
 
-
-/* Explicitly initialize this module.  */
 gcry_err_code_t
 _gcry_md_init (void)
 {
@@ -1300,41 +1251,3 @@ gcry_md_list (int *list, int *list_length)
 
   return err;
 }
-
-
-/* Run the selftests for digest algorithm ALGO with optional reporting
-   function REPORT.  */
-gpg_error_t
-_gcry_md_selftest (int algo, int extended, selftest_report_func_t report)
-{
-  gcry_module_t module = NULL;
-  cipher_extra_spec_t *extraspec = NULL;
-  gcry_err_code_t ec = 0;
-
-  REGISTER_DEFAULT_DIGESTS;
-
-  ath_mutex_lock (&digests_registered_lock);
-  module = _gcry_module_lookup_id (digests_registered, algo);
-  if (module && !(module->flags & FLAG_MODULE_DISABLED))
-    extraspec = module->extraspec;
-  ath_mutex_unlock (&digests_registered_lock);
-  if (extraspec && extraspec->selftest)
-    ec = extraspec->selftest (algo, extended, report);
-  else
-    {
-      ec = GPG_ERR_DIGEST_ALGO;
-      if (report)
-        report ("digest", algo, "module", 
-                module && !(module->flags & FLAG_MODULE_DISABLED)?
-                "no selftest available" :
-                module? "algorithm disabled" : "algorithm not found");
-    }
-
-  if (module)
-    {
-      ath_mutex_lock (&digests_registered_lock);
-      _gcry_module_release (module);
-      ath_mutex_unlock (&digests_registered_lock);
-    }
-  return gpg_error (ec);
-}
